
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rif_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

typedef struct List {
		char		*path;
		int		Ox,Oy,Nx,Ny;
		struct List	*next;
	} List;

static List	*Peep(char *path,int Ox,int Oy,List *next,
		      int *x1,int *y1,int *x2,int *y2)
{
	List	*list;
	int	x,y;

	if ((list=(List *)malloc(sizeof(List)))==NULL ||
	    (list->path=strdup(path))==NULL)
	    Error("list memory allocation error.");

	(void)ReadImageFile_Float(path,&(list->Nx),&(list->Ny),NULL,NULL);

	x=(list->Ox=Ox)+list->Nx-1;
	y=(list->Oy=Oy)+list->Ny-1;
	if ((list->next=next)==NULL) {
	    *x1=Ox; *x2=x;
	    *y1=Oy; *y2=y;
	}
	else {
	    if (*x1>Ox) *x1=Ox; if (*x2<x) *x2=x;
	    if (*y1>Oy) *y1=Oy; if (*y2<y) *y2=y;
	}
	return list;
}

#define LEN	2048

static void	Read(char *path,
		     int *Nx,int *Ny,FOM ***fom,double *base,double *step)
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	f1,f2;

	if ((bps=ReadImageFile_Float(path,Nx,Ny,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&f1)!=1 ||
			sscanf(str[n-1],"%lf",&f2)!=1 || f1>=f2)
			u_c=u;
		    else {
			u_c=c; *base=f1; *step=(f2-f1)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",base)!=1 ||
			 sscanf(str[n-1],"%lf",step)!=1 || *step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    *base=0.0; *step=1.0;
	}
}

#ifndef	NUM
#define NUM	unsigned short
#endif

int	main(int argc,char **argv)
{
	int	area,i,x1,y1,x2,y2,Ox,Oy,Nx,Ny,y,x,x0,y0;
	List	*next,*list;
	char	line[LEN],path[LEN];
	NUM	**num;
	FOM	**sum,**fom;
	double	base,step,none;

	if ((area=argc%3)==2 || argc==1 || argc==4) Error(
	    "usage : add_2d_float {(TIFF Ox Oy) ...} none {x1 y1 x2 y2} TIFF"
	);
	next=NULL;
	if ((i=argc-area*4-3)!=0)
	    while ((i-=3)>=0)
		next=Peep(argv[i+1],atoi(argv[i+2]),atoi(argv[i+3]),next,
			  &x1,&y1,&x2,&y2);
	else {
	    while (fgets(line,LEN,stdin)!=NULL)
		if ((i=sscanf(line,"%s %d %d",path,&Ox,&Oy))>=1) {
		    if (i!=3) Ox=Oy=0;

		    next=Peep(path,Ox,Oy,next,&x1,&y1,&x2,&y2);
		}

	    if (next==NULL) Error("no image.");
	}
	if (area==1) {
	    if ((Ox=atoi(argv[argc-5]))>
		(Nx=atoi(argv[argc-3])) || Ox>x2 || Nx<x1 ||
		(Oy=atoi(argv[argc-4]))>
		(Ny=atoi(argv[argc-2])) || Oy>y2 || Ny<y1) Error("bad area.");

	    Nx-=(Ox-1); Ny-=(Oy-1);
	}
	else {
	    Nx=x2-(Ox=x1)+1; Ny=y2-(Oy=y1)+1;
	}
	if ((num=(NUM **)malloc(sizeof(NUM *)*Ny))==NULL ||
	    (*num=(NUM *)malloc(sizeof(NUM)*Ny*Nx))==NULL ||
	    (sum=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
	    (*sum=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL)
	    Error("image memory allocation error.");

	for (y=0; y<Ny; y++) {
	    if (y>0) {
		num[y]=num[y-1]+Nx; sum[y]=sum[y-1]+Nx;
	    }
	    for (x=0; x<Nx; x++) sum[y][x]=(FOM)(num[y][x]=0);
	}
	while ((list=next)!=NULL) {
	    if ((x0=Ox-list->Ox)+Nx>0 && x0<list->Nx &&
		(y0=Oy-list->Oy)+Ny>0 && y0<list->Ny) {
		Read(list->path,&x,&y,&fom,&base,&step);

		if (x!=list->Nx || y!=list->Ny) {
		    (void)fputs(list->path,stderr);
		    Error(" : image size not match.");
		}
		x1=(x0>=0)?0:-x0; if ((x2=list->Nx-x0)>Nx) x2=Nx;
		y1=(y0>=0)?0:-y0; if ((y2=list->Ny-y0)>Ny) y2=Ny;
		for (y=y1; y<y2; y++)
		for (x=x1; x<x2; x++) {
		    ++num[y][x]; sum[y][x]+=(base+step*fom[y+y0][x+x0]);
		}
		for (y=0; y<list->Ny; y++) free(fom[y]); free(fom);
	    }
	    free(list->path); next=list->next; free(list);
	}
	none=atof(argv[argc-area*4-2]);
	for (y=0; y<Ny; y++)
	for (x=0; x<Nx; x++)
	    sum[y][x]=(num[y][x]==0)?none:sum[y][x]/(double)num[y][x];

	StoreImageFile_Float(argv[argc-1],Nx,Ny,sum,SIF_F_desc);
	(void)puts(SIF_F_desc);

	free(*sum); free(sum); free(*num); free(num);

	return 0;
}
