
#include <stdio.h>
#include <stdlib.h>
#include "rif_f.h"
#include "csi_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

typedef struct List {
		char		**paths;
		int		Ox,Oy,Oz,Nx,Ny,Nz;
		struct List	*next;
	} List;

static List	*Peep(char *dir,char *NF,int Ox,int Oy,int Oz,List *next,
		      int *x1,int *y1,int *z1,int *x2,int *y2,int *z2)
{
	List	*list;
	int	x,y,z;

	if ((list=(List *)malloc(sizeof(List)))==NULL)
	    Error("list memory allocation error.");

	list->paths=
	CheckSliceImages_Float(dir,NF,&(list->Nx),&(list->Ny),&(list->Nz),NULL);

	x=(list->Ox=Ox)+list->Nx-1;
	y=(list->Oy=Oy)+list->Ny-1;
	z=(list->Oz=Oz)+list->Nz-1;
	if ((list->next=next)==NULL) {
	    *x1=Ox; *x2=x;
	    *y1=Oy; *y2=y;
	    *z1=Oz; *z2=z;
	}
	else {
	    if (*x1>Ox) *x1=Ox; if (*x2<x) *x2=x;
	    if (*y1>Oy) *y1=Oy; if (*y2<y) *y2=y;
	    if (*z1>Oz) *z1=Oz; if (*z2<z) *z2=z;
	}
	return list;
}

#define LEN	2048

static void	Read(char *path,
		     int *Nx,int *Ny,FOM ***fom,double *base,double *step)
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	f1,f2;

	if ((bps=ReadImageFile_Float(path,Nx,Ny,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&f1)!=1 ||
			sscanf(str[n-1],"%lf",&f2)!=1 || f1>=f2)
			u_c=u;
		    else {
			u_c=c; *base=f1; *step=(f2-f1)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",base)!=1 ||
			 sscanf(str[n-1],"%lf",step)!=1 || *step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    *base=0.0; *step=1.0;
	}
}

#ifndef	NUM
#define NUM	unsigned short
#endif

int	main(int argc,char **argv)
{
	int	area,i,x1,y1,z1,x2,y2,z2,Ox,Oy,Oz,Nx,Ny,Nz,y,z,x,z0,X,Y,x0,y0;
	List	*next,*list;
	char	line[LEN],dir[LEN],NF[LEN],form[LEN],path[LEN];
	NUM	**num;
	FOM	**sum,**fom;
	double	none,base,step;

	if ((area=argc%5)<3 || argc==4) Error(
"usage : add_3d_float {(dir/ NF Ox Oy Oz) ...} none {x1 y1 z1 x2 y2 z2} dir/"
	);
	next=NULL;
	if ((i=argc-(area/=4)*6-3)!=0)
	    while ((i-=5)>=0)
		next=Peep(argv[i+1],argv[i+2],
			  atoi(argv[i+3]),atoi(argv[i+4]),atoi(argv[i+5]),
			  next,
			  &x1,&y1,&z1,&x2,&y2,&z2);
	else {
	    while (fgets(line,LEN,stdin)!=NULL)
		if ((i=sscanf(line,"%s %s %d %d %d",dir,NF,&Ox,&Oy,&Oz))>=2) {
		    if (i!=5) Ox=Oy=Oz=0;

		    next=Peep(dir,NF,Ox,Oy,Oz,next,&x1,&y1,&z1,&x2,&y2,&z2);
		}

	    if (next==NULL) Error("no image.");
	}
	if (area==1) {
	    if ((Ox=atoi(argv[argc-7]))>
		(Nx=atoi(argv[argc-4])) || Ox>x2 || Nx<x1 ||
		(Oy=atoi(argv[argc-6]))>
		(Ny=atoi(argv[argc-3])) || Oy>y2 || Ny<y1 ||
		(Oz=atoi(argv[argc-5]))>
		(Nz=atoi(argv[argc-2])) || Oz>z2 || Nz<z1) Error("bad area.");

	    Nx-=(Ox-1); Ny-=(Oy-1); Nz-=(Oz-1);
	}
	else {
	    Nx=x2-(Ox=x1)+1; Ny=y2-(Oy=y1)+1; Nz=z2-(Oz=z1)+1;
	}
	if ((num=(NUM **)malloc(sizeof(NUM *)*Ny))==NULL ||
	    (*num=(NUM *)malloc(sizeof(NUM)*Ny*Nx))==NULL ||
	    (sum=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
	    (*sum=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL)
	    Error("image memory allocation error.");

	for (y=1; y<Ny; y++) {
	    num[y]=num[y-1]+Nx; sum[y]=sum[y-1]+Nx;
	}
	none=atof(argv[argc-area*6-2]);

	i=1; for (z=10; z<Nz; z*=10) ++i;
	(void)sprintf(form,"%s/%%0%dd.tif",argv[argc-1],i);

	for (z=0; z<Nz; z++) {
	    for (y=0; y<Ny; y++)
	    for (x=0; x<Nx; x++) sum[y][x]=(FOM)(num[y][x]=0);

	    for (list=next; list!=NULL; list=list->next)
		if ((x0=Ox-list->Ox)+Nx>0 && x0<list->Nx &&
		    (y0=Oy-list->Oy)+Ny>0 && y0<list->Ny &&
		    (z0=Oz-list->Oz+z)>=0 && z0<list->Nz) {
		    Read(list->paths[z0],&X,&Y,&fom,&base,&step);

		    if (X>list->Nx || Y>list->Ny) {
			(void)fputs(list->paths[z0],stderr);
			Error(" : image size not match.");
		    }
		    x1=(x0>=0)?0:-x0; if ((x2=X-x0)>Nx) x2=Nx;
		    y1=(y0>=0)?0:-y0; if ((y2=Y-y0)>Ny) y2=Ny;
		    for (y=y1; y<y2; y++)
		    for (x=x1; x<x2; x++) {
			++num[y][x]; sum[y][x]+=(base+step*fom[y+y0][x+x0]);
		    }
		    for (y=0; y<Y; y++) free(fom[y]); free(fom);
		}

	    for (y=0; y<Ny; y++)
	    for (x=0; x<Nx; x++)
		sum[y][x]=(num[y][x]==0)?none:sum[y][x]/(double)num[y][x];

	    (void)sprintf(path,form,z);
	    StoreImageFile_Float(path,Nx,Ny,sum,SIF_F_desc);
	    (void)printf("%d\t%s\n",z,SIF_F_desc);
	}
	free(*sum); free(sum); free(*num); free(num);

	while ((list=next)!=NULL) {
	    for (z=0; z<list->Nz; z++) free(list->paths[z]); 

	    free(list->paths); next=list->next; free(list);
	}
	return 0;
}
