
#include <stdio.h>
#include <stdlib.h>
#include "rif_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

static int	Range(double x,double y,double *min)
{
	double	max,xy=x+y;

	*min=max=0.0; if (x <*min) *min=x;  else if (x >max) max=x;
		      if (y <*min) *min=y;  else if (y >max) max=y;
		      if (xy<*min) *min=xy; else if (xy>max) max=xy;

	return (int)(max-(*min)+0.5);
}

#define LEN	2048

int	main(int argc,char **argv)
{
	int	Nx,Ny,Nh,Nv,j,i;
	FOM	**org,**new;
	double	hx,hy,vx,vy,det,h0,v0,pv,xh,xv,yh,yv,x,y;

#ifdef	AFFINE
#define EXE	"affine_2d_float"
#else
#define EXE	"bli_float"
#endif
	if (argc!=6 && argc!=8 && argc!=10 && argc!=12)
	    Error("usage : " EXE " org.tif hx hy vx vy\n"
		  "        " EXE " org.tif hx hy vx vy PV new.tif\n"
		  "        " EXE " org.tif hx hy vx vy h0 v0 PV new.tif\n"
		  "        " EXE " org.tif hx hy vx vy h0 v0 Nh Nv PV new.tif");
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	min,max,b,s;

	if ((bps=ReadImageFile_Float(argv[1],&Nx,&Ny,&org,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&min)!=1 ||
			sscanf(str[n-1],"%lf",&max)!=1 || min>=max)
			u_c=u;
		    else {
			u_c=c; b=min; s=(max-min)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",&b)!=1 ||
			 sscanf(str[n-1],"%lf",&s)!=1 || s<=0.0)?u:c;

		if (argc!=6 && u_c==c)
		    for (j=0; j<Ny; j++)
		    for (i=0; i<Nx; i++) org[j][i]=b+s*org[j][i];

		free(id);
	    }
#ifdef	U_C
	    else
		u_c=u;

	    (void)fprintf(stderr,"%s : %s (warning).\n",argv[1],u_c);
#endif
	}
	else
	    if (id!=NULL) free(id);
}

#define X	(double)Nx
#define Y	(double)Ny

	hx=atof(argv[2]); hy=atof(argv[3]);
	vx=atof(argv[4]); vy=atof(argv[5]); det=hx*vy-hy*vx;
	if (-0.5<X*Y*det && X*Y*det<0.5) Error("bad transfer matrix.");

	if (argc==6 || argc==8) {
	    Nh=Range(hx*X,hy*Y,&h0);
	    Nv=Range(vx*X,vy*Y,&v0);

	    if (argc==6) {
		(void)printf("%lf\t%lf\t%d\t%d\n",h0,v0,Nh,Nv); goto Exit;
	    }
	}
	else {
	    h0=atof(argv[6]);
	    v0=atof(argv[7]);

	    if (argc==10) {
		Nh=Nx;
		Nv=Ny;
	    }
	    else
		if ((Nh=atoi(argv[8]))<=0 ||
		    (Nv=atoi(argv[9]))<=0) Error("bad image size.");
	}
	if ((new=(FOM **)malloc(sizeof(FOM *)*Nv))==NULL ||
	    (*new=(FOM *)malloc(sizeof(FOM)*Nv*Nh))==NULL)
	    Error("memory allocation error.");

	for (j=1; j<Nv; j++) new[j]=new[j-1]+Nh;

	pv=atof(argv[argc-2]);

	h0+=0.5; xh=vy/det; yh=(-vx)/det;
	v0+=0.5; xv=(-hy)/det; yv=hx/det;
	for (vx=xv*v0, vy=yv*v0, j=0; j<Nv; j++, vx+=xv, vy+=yv)
	for (hx=xh*h0, hy=yh*h0, i=0; i<Nh; i++, hx+=xh, hy+=yh)
	    if ((x=hx+vx)<0.0 || x>=X || (y=hy+vy)<0.0 || y>=Y)
		new[j][i]=pv;
	    else
#ifdef	AFFINE
		new[j][i]=org[(int)y][(int)x];
#else
{
	int	x1,x2,y1,y2;
	double	o,ox,oy;

	if ((x+=0.5)<1.0)
	    x1=x2=0;
	else if (x<X) {
	    x1=(x2=(int)x)-1; x-=(double)x2;
	}
	else {
	    x1=x2=Nx-1; x-=X;
	}
	if ((y+=0.5)<1.0)
	    y1=y2=0;
	else if (y<Y) {
	    y1=(y2=(int)y)-1; y-=(double)y2;
	}
	else {
	    y1=y2=Ny-1; y-=Y;
	}
	o=org[y1][x1]; ox=org[y1][x2]-o; oy=org[y2][x1]-o;
	new[j][i]=(org[y2][x2]-ox-oy-o)*x*y+ox*x+oy*y+o;
}
#endif
	StoreImageFile_Float(argv[argc-1],Nh,Nv,new,SIF_F_desc);
	(void)puts(SIF_F_desc);

	free(*new); free(new);
Exit:;
	for (j=0; j<Ny; j++) free(org[j]); free(org);

	return 0;
}
