
#include <stdio.h>
#include <stdlib.h>
#include "rif_f.h"
#include "csi_f.h"

extern void	Error(char *msg);

#define LEN	2048

static void	Read(char *path,
		     int *Nx,int *Ny,FOM ***fom,double *base,double *step)
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	f1,f2;

	if ((bps=ReadImageFile_Float(path,Nx,Ny,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&f1)!=1 ||
			sscanf(str[n-1],"%lf",&f2)!=1 || f1>=f2)
			u_c=u;
		    else {
			u_c=c; *base=f1; *step=(f2-f1)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",base)!=1 ||
			 sscanf(str[n-1],"%lf",step)!=1 || *step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    *base=0.0; *step=1.0;
	}
}

#define EPS	1e-9

int	main(int argc,char **argv)
{
	if (argc==2)
{
	int	Nx,Ny,y,x;
	FOM	**fom;
	double	base,step,m,M=0.0,X=0.0,Y=0.0;

	Read(argv[1],&Nx,&Ny,&fom,&base,&step);

	(void)printf("%d\t%d\n",Nx,Ny);

	for (y=0; y<Ny; y++) {
	    for (x=0; x<Nx; x++) {
		M+=(m=base+step*fom[y][x]); X+=(m*(double)x);
					    Y+=(m*(double)y);
	    }
	    free(fom[y]);
	}
	free(fom);

	(void)printf("%lf\n",M);
	if (M<-EPS || M>EPS) (void)printf("%lg\t%lg\n",X/M+0.5,Y/M+0.5);
}
	else if (argc==3)
{
	char	**path;
	int	Nx,Ny,Nz,z,y,x;
	FOM	**fom;
	double	base,step,m,M=0.0,X=0.0,Y=0.0,Z=0.0;

	path=CheckSliceImages_Float(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

	(void)printf("%d\t%d\t%d\n",Nx,Ny,Nz);

	for (z=0; z<Nz; z++) {
	    Read(path[z],&Nx,&Ny,&fom,&base,&step);

	    for (y=0; y<Ny; y++) {
		for (x=0; x<Nx; x++) {
		    M+=(m=base+step*fom[y][x]); X+=(m*(double)x);
						Y+=(m*(double)y);
						Z+=(m*(double)z);
		}
		free(fom[y]);
	    }
	    free(fom); free(path[z]);
	}
	free(path);

	(void)printf("%lf\n",M);
	if (M<-EPS || M>EPS) printf("%lg\t%lg\t%lg\n",X/M+0.5,Y/M+0.5,Z/M+0.5);
}
	else
	    Error("usage : gc_float TIFF\n"
		  "        gc_float directory nameFile");

	return 0;
}
