
#include <stdio.h>
#include <stdlib.h>
#include "rif_f.h"
#include "csi_f.h"
#include "sif_f.h"

#ifdef	LOG
#include <math.h>
#endif

extern void	Error(char *msg);

#define LEN	2048

static void	Read(char *path,int *Nh,int *Nv,FOM ***fom)
{
	int	bps,n,H,V,v,h;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	min,max,b,s;
	FOM	**F;

	if ((bps=ReadImageFile_Float(path,Nh,Nv,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&min)!=1 ||
			sscanf(str[n-1],"%lf",&max)!=1 || min>=max)
			u_c=u;
		    else {
			u_c=c; b=min; s=(max-min)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",&b)!=1 ||
			 sscanf(str[n-1],"%lf",&s)!=1 || s<=0.0)?u:c;

		if (u_c==c) {
		    H=(*Nh);
		    V=(*Nv); F=(*fom); for (v=0; v<V; v++)
				       for (h=0; h<H; h++) F[v][h]=b+s*F[v][h];
		}
		free(id);
	    }
#ifdef	U_C
	    else
		u_c=u;

	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else
	    if (id!=NULL) free(id);
}

#define EPS	1e-9

int	main(int argc,char **argv)
{
	char	**Px,**Py,**Pz,form[LEN],path[LEN];
	int	Hx,Vx,Dx,Hy,Vy,Dy,Hz,Vz,Dz,Nx,Ny,Nz,z,y,x,Nd,d,Nh,Nv,v,h,len;
	double	Sx,Sy,Sz,Bx,By,Bz;
	FOM	***C,**Fx,**Fy,**Fz,fx,fy,fz;

	if (argc!=17)
	    Error("usage : hg_trio_float xDir/ xNF xBase xStep xBins \\\n"
		  "                      yDir/ yNF yBase yStep yBins \\\n"
		  "                      zDir/ zNF zBase zStep zBins HG/");

	Px=CheckSliceImages_Float(argv[ 1],argv[ 2],&Hx,&Vx,&Dx,NULL);
	Py=CheckSliceImages_Float(argv[ 6],argv[ 7],&Hy,&Vy,&Dy,NULL);
	Pz=CheckSliceImages_Float(argv[11],argv[12],&Hz,&Vz,&Dz,NULL);
	if (Hx!=Hy || Hy!=Hz || Vx!=Vy || Vy!=Vz || Dx!=Dy || Dy!=Dz)
	    (void)fputs("image size not match (warning).\n",stderr);

	if ((Nx=atoi(argv[ 5]))<=0) Error("bad number of bins along x.");
	if ((Ny=atoi(argv[10]))<=0) Error("bad number of bins along y.");
	if ((Nz=atoi(argv[15]))<=0) Error("bad number of bins along z.");

	if ((Sx=atof(argv[ 4]))<EPS) Error("bad value of step along x.");
	if ((Sy=atof(argv[ 9]))<EPS) Error("bad value of step along y.");
	if ((Sz=atof(argv[14]))<EPS) Error("bad value of step along z.");

	if ((C=(FOM ***)malloc(sizeof(FOM **)*Nz))==NULL ||
	    (*C=(FOM **)malloc(sizeof(FOM *)*Nz*Ny))==NULL ||
	    (**C=(FOM *)malloc(sizeof(FOM)*Nz*Ny*Nx))==NULL)
	    Error("memory allocation error.");

	for (z=0; z<Nz; z++) {
	    if (z>0) C[z]=C[z-1]+Ny;

	    for (y=0; y<Ny; y++) {
		if (z>0 || y>0) C[z][y]=C[z][y-1]+Nx;

		for (x=0; x<Nx; x++) C[z][y][x]=0.0;
	    }
	}
	Bx=atof(argv[ 3])-Sx/2.0;
	By=atof(argv[ 8])-Sy/2.0;
	Bz=atof(argv[13])-Sz/2.0; Nd=Dx; if (Nd>Dy) Nd=Dy; if (Nd>Dz) Nd=Dz;
	for (d=0; d<Nd; d++) {
	    Read(Px[d],&Hx,&Vx,&Fx);
	    Read(Py[d],&Hy,&Vy,&Fy);
	    Read(Pz[d],&Hz,&Vz,&Fz); Nh=Hx; if (Nh>Hy) Nh=Hy; if (Nh>Hz) Nh=Hz;
				     Nv=Vx; if (Nv>Vy) Nv=Vy; if (Nv>Vz) Nv=Vz;
	    for (v=0; v<Nv; v++)
	    for (h=0; h<Nh; h++)
	    if ((fx=Fx[v][h])>=Bx && (x=(int)((fx-Bx)/Sx))<Nx &&
		(fy=Fy[v][h])>=By && (y=(int)((fy-By)/Sy))<Ny &&
		(fz=Fz[v][h])>=Bz && (z=(int)((fz-Bz)/Sz))<Nz) C[z][y][x]+=1.0;

	    for (v=0; v<Vz; v++) free(Fz[v]); free(Fz);
	    for (v=0; v<Vy; v++) free(Fy[v]); free(Fy);
	    for (v=0; v<Vx; v++) free(Fx[v]); free(Fx);
	}
	for (d=0; d<Dz; d++) free(Pz[d]); free(Pz);
	for (d=0; d<Dy; d++) free(Py[d]); free(Py);
	for (d=0; d<Dx; d++) free(Px[d]); free(Px);

	len=1; for (z=10; z<Nz; z*=10) ++len;
	(void)sprintf(form,"%s/%%0%dd.tif",argv[16],len);

	for (z=0; z<Nz; z++) {
#ifdef	LOG
	    for (y=0; y<Ny; y++)
	    for (x=0; x<Nx; x++)
		if (C[z][y][x]>0.0) C[z][y][x]=LOG(C[z][y][x])+1.0;
#endif
	    (void)sprintf(path,form,z);
	    StoreImageFile_Float(path,Nx,Ny,C[z],SIF_F_desc);
	    (void)printf("%d\t%s\n",z,SIF_F_desc);
	}
	free(**C); free(*C); free(C);

	return 0;
}
