
#include <stdio.h>
#include <stdlib.h>
#include "sif_f.h"

#ifdef	TXY
#define log(id_td)	(1.0/(id_td))

#define T_P		"t"
#else
#include <math.h>

#define T_P		"p"
#endif

static void	Abort(char *path,char *msg)
{
	(void)fprintf(stderr,"%s : %s.\n",path,msg); exit(1);
}

static int	FGetW(FILE *file)
{
	int	l,h;

	return ((l=fgetc(file))==EOF || (h=fgetc(file))==EOF)?-1:(h<<8)|l;
}

static FILE	*Open(char *path,int check,int *Nx,int *Ny)
{
	FILE	*file=fopen(path,"rb");
	int	len,idx;

	if (file==NULL) Abort(path,"file not found");

	if (fgetc(file)!='I' ||
	    fgetc(file)!='M') Abort(path,"bad magic number");

	if ((len=FGetW(file))<0) Abort(path,"bad comment length");

	if (check) {
	    if (*Nx!=FGetW(file) ||
		*Ny!=FGetW(file)) Abort(path,"image size not match");
	}
	else
	    if ((*Nx=FGetW(file))<=0 ||
		(*Ny=FGetW(file))<=0) Abort(path,"bad image size");

	for (idx=4; idx>0; idx--) (void)fgetc(file);

	if (FGetW(file)!=2) Abort(path,"bad image type");

	for (idx=50+len; idx>0; idx--) (void)fgetc(file);

	return file;
}

static int	FGetI(char *path,FILE *file)
{
	int	i;

	if ((i=FGetW(file))<0) Abort(path,"unexpected end of file");

	return i;
}

static void	Error(char *msg)
{
	(void)fputs(msg,stderr); (void)fputc('\n',stderr); exit(1);
}

#define NONE	0.0

int	main(int argc,char **argv)
{
	FILE	*D,*I,*T;
	int	Nx,Ny,y,x,d,i,t;
	FOM	**fom;

	if (argc!=5)
	    Error("usage : hp3" T_P "xy_float d.img i.img t.img " T_P "xy.tif");

	D=Open(argv[1],0,&Nx,&Ny);
	I=Open(argv[2],1,&Nx,&Ny);
	T=Open(argv[3],1,&Nx,&Ny);

	if ((fom=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
	    (*fom=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL)
	    Error("memory allocation error.");

	for (y=0; y<Ny; y++) {
	    if (y>0) fom[y]=fom[y-1]+Nx;

	    for (x=0; x<Nx; x++) {
		d=FGetI(argv[1],D);
		i=FGetI(argv[2],I);
		t=FGetI(argv[3],T);
		fom[y][x]=(i>d && t>d)?log((double)(i-d)/(double)(t-d)):NONE;
	    }
	}
	(void)fclose(T); (void)fclose(I); (void)fclose(D);

	StoreImageFile_Float(argv[4],Nx,Ny,fom,SIF_F_desc);
	(void)puts(SIF_F_desc);

	free(*fom); free(fom);

	return 0;
}
