
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rif_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

#define LEN	2048

static int	Nx,Ny,S0=0;
static FOM	**S1,**S2;

static void	S012(char *path)
{
	int	bps,nx,ny,n,y,x;
	FOM	**fom;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	min,max,base,step;

	if ((bps=ReadImageFile_Float(path,&nx,&ny,&fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&min)!=1 ||
			sscanf(str[n-1],"%lf",&max)!=1 || min>=max)
			u_c=u;
		    else {
			u_c=c; base=min; step=(max-min)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",&base)!=1 ||
			 sscanf(str[n-1],"%lf",&step)!=1 || step<=0.0)?u:c;

		if (u_c==c)
		    for (y=0; y<ny; y++)
		    for (x=0; x<nx; x++) fom[y][x]=base+step*fom[y][x];

		free(id);
	    }
#ifdef	U_C
	    else
		u_c=u;

	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else
	    if (id!=NULL) free(id);

	if (++S0==1) {
	    Nx=nx; Ny=ny;

	    if ((S1=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
		(*S1=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL ||
		(S2=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
		(*S2=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL)
		Error("memory allocation error.");

	    for (y=0; y<Ny; y++) {
		if (y>0) {
		    S1[y]=S1[y-1]+Nx; S2[y]=S2[y-1]+Nx;
		}
		for (x=0; x<Nx; x++) S1[y][x]=S2[y][x]=0.0;
	    }
	}
	else
	    if (nx!=Nx || ny!=Ny) {
		(void)fputs(path,stderr); Error(" : image size not match.");
	    }

	for (y=0; y<Ny; y++) {
	    for (x=0; x<Nx; x++) {
		S1[y][x]+=fom[y][x]; S2[y][x]+=(fom[y][x]*fom[y][x]);
	    }
	    free(fom[y]);
	}
	free(fom);
}

int	main(int argc,char **argv)
{
	char	path[LEN];
	int	y,x;

	if (argc<3) Error("usage : m+sd_float {TIFFs} mean.tif SD.tif");

	if (argc==3)
	    while (scanf("%s",path)==1) S012(path);
	else
	    while (--argc>=3) S012(*(++argv));

	if (S0<2) Error("too few TIFF.");

	for (y=0; y<Ny; y++)
	for (x=0; x<Nx; x++) {
	    S1[y][x]/=(double)S0;
	    S2[y][x]=sqrt(S2[y][x]/(double)S0-S1[y][x]*S1[y][x]);
	}
	StoreImageFile_Float(argv[1],Nx,Ny,S1,SIF_F_desc);
	StoreImageFile_Float(argv[2],Nx,Ny,S2,SIF_F_desc);

	free(*S2); free(S2); free(*S1); free(S1);

	return 0;
}
