
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rif_f.h"
#include "csi_f.h"

extern void	Error(char *msg);

#define LEN	2048

static void	Read(char *path,
		     int *Nx,int *Ny,FOM ***fom,double *base,double *step)
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	f1,f2;

	if ((bps=ReadImageFile_Float(path,Nx,Ny,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&f1)!=1 ||
			sscanf(str[n-1],"%lf",&f2)!=1 || f1>=f2)
			u_c=u;
		    else {
			u_c=c; *base=f1; *step=(f2-f1)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",base)!=1 ||
			 sscanf(str[n-1],"%lf",step)!=1 || *step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    *base=0.0; *step=1.0;
	}
}

#ifndef	M_PI
#define M_PI	3.14159265358979323846
#endif

#define EPS	1e-9

void	OvalFitting(char *path,double Ux,double Uy)
{
	double	M=0.0,Mx=0.0,My=0.0,Mxx=0.0,Myy=0.0,Mxy=0.0;
{
	int	Nx,Ny,y,x;
	FOM	**fom;
	double	base,step,m;

	Read(path,&Nx,&Ny,&fom,&base,&step);

	(void)printf("%d\t%d\n",Nx,Ny);

	for (y=0; y<Ny; y++) {
	    for (x=0; x<Nx; x++) {
		M+=(m=base+step*fom[y][x]);
		Mx+=(m*(double)x); Mxx+=(m*(double)x*(double)x);
		My+=(m*(double)y); Myy+=(m*(double)y*(double)y);
				   Mxy+=(m*(double)x*(double)y);
	    }
	    free(fom[y]);
	}
	free(fom);
}
	(void)printf("%lf\n",M);

	if (fabs(M)>EPS)
{
	double	x=Mx/M,
		y=My/M,
		xx=Ux*Ux*(Mxx-M*x*x+M/12.0),
		yy=Uy*Uy*(Myy-M*y*y+M/12.0),
		xy=Ux*Uy*(Mxy-M*x*y),
		a=(xx-yy)/2.0,
		b=(xx+yy)/2.0,
		c=sqrt(a*a+xy*xy);

	(void)printf("%lg\t%lg\n%lg\n%lg\t%lg\n",
		     x+1.0/2.0,y+1.0/2.0,
		     fmod(3.0/4.0*M_PI-atan2(a,xy)/2.0,M_PI)*180.0/M_PI,
		     2.0*sqrt((b-c)/M),2.0*sqrt((b+c)/M));
}}

static double	XX,YY,ZZ,XY,YZ,ZX,L0_Lc;
static int	X0,Y0,Z0,X1,Y1,Z1,X2,Y2,Z2,X3,Y3,Z3;

static void	Test(int x1,int y1,int z1,
		     int x2,int y2,int z2,
		     int x3,int y3,int z3)
{
	int	x0=x1+x2+x3,
		y0=y1+y2+y3,
		z0=z1+z2+z3;
	double	x=(double)x0,xx=x*x,
		y=(double)y0,yy=y*y,
		z=(double)z0,zz=z*z,
		l0_lc=(XX*xx+YY*yy+ZZ*zz+XY*x*y+YZ*y*z+ZX*z*x)/(xx+yy+zz);

	if (l0_lc>=L0_Lc) {
	    X0=x0; Y0=y0; Z0=z0; X1=x1; Y1=y1; Z1=z1;
				 X2=x2; Y2=y2; Z2=z2;
				 X3=x3; Y3=y3; Z3=z3; L0_Lc=l0_lc;
	}
}

static void	Recursion(int recursions,int x1,int y1,int z1,
					 int x2,int y2,int z2,
					 int x3,int y3,int z3)
{
	if (--recursions<0)
	    Test(x1,y1,z1,x2,y2,z2,x3,y3,z3);
	else
{
	int	x4=x1+x2,y4=y1+y2,z4=z1+z2,
		x5=x2+x3,y5=y2+y3,z5=z2+z3,
		x6=x3+x1,y6=y3+y1,z6=z3+z1;

	Recursion(recursions,x1*2,y1*2,z1*2,x4,y4,z4,x6,y6,z6);
	Recursion(recursions,x2*2,y2*2,z2*2,x5,y5,z5,x4,y4,z4);
	Recursion(recursions,x3*2,y3*2,z3*2,x6,y6,z6,x5,y5,z5);
	Recursion(recursions,x4  ,y4  ,z4  ,x5,y5,z5,x6,y6,z6);
}}

#ifndef	RECURSIONS
#define RECURSIONS	7
#endif

#ifndef	__DBL_MANT_DIG__
#define __DBL_MANT_DIG__	53
#endif

void	OvoidFitting(char *D,char *NF,double Ux,double Uy,double Uz)
{
	double	M=0.0,Mx=0.0,My=0.0,Mz=0.0,Mxx=0.0,Myy=0.0,Mzz=0.0,
					   Mxy=0.0,Myz=0.0,Mzx=0.0;
{
	char	**path;
	int	Nx,Ny,Nz,z,y,x;
	FOM	**fom;
	double	base,step,m;

	path=CheckSliceImages_Float(D,NF,&Nx,&Ny,&Nz,NULL);

	(void)printf("%d\t%d\t%d\n",Nx,Ny,Nz);

	for (z=0; z<Nz; z++) {
	    Read(path[z],&Nx,&Ny,&fom,&base,&step);

	    for (y=0; y<Ny; y++) {
		for (x=0; x<Nx; x++) {
		    M+=(m=base+step*fom[y][x]);
		    Mx+=(m*(double)x); Mxx+=(m*(double)x*(double)x);
		    My+=(m*(double)y); Myy+=(m*(double)y*(double)y);
		    Mz+=(m*(double)z); Mzz+=(m*(double)z*(double)z);
				       Mxy+=(m*(double)x*(double)y);
				       Myz+=(m*(double)y*(double)z);
				       Mzx+=(m*(double)z*(double)x);
		}
		free(fom[y]);
	    }
	    free(fom); free(path[z]);
	}
	free(path);
}
	(void)printf("%lf\n",M);

	if (fabs(M)>EPS)
{
	double	x,y,z,x0,y0,z0,rp,q,cl,sl,r,cp,sp,ccl,ssl,csl,hh,vv,hv,a,b,c;
	int	i,x1,y1,z1,x2,y2,z2,x3,y3,z3,x4,y4,z4,x5,y5,z5,x6,y6,z6;

	x=Mx/M; XX=Ux*Ux*(Mxx-M*x*x+M/12.0);
	y=My/M; YY=Uy*Uy*(Myy-M*y*y+M/12.0); XY=Ux*Uy*(Mxy-M*x*y)*2.0;
	z=Mz/M; ZZ=Uz*Uz*(Mzz-M*z*z+M/12.0); YZ=Uy*Uz*(Myz-M*y*z)*2.0;
					     ZX=Uz*Ux*(Mzx-M*z*x)*2.0;

	L0_Lc=0.0-(fabs(XX)+fabs(YY)+fabs(ZZ)+fabs(XY)+fabs(YZ)+fabs(ZX));

	Recursion(RECURSIONS,0,0,1,-1, 0,0, 0,-1,0);
	Recursion(RECURSIONS,0,0,1, 0,-1,0, 1, 0,0);
	Recursion(RECURSIONS,0,0,1, 1, 0,0, 0, 1,0);
	Recursion(RECURSIONS,0,0,1, 0, 1,0,-1, 0,0);

	for (i=RECURSIONS+1; i<(__DBL_MANT_DIG__-1-2)/2; i++) {
	    x1=(X0*=2); x2=X2; x3=X3; x4=X1+X2; x5=X2+X3; x6=X3+X1;
	    y1=(Y0*=2); y2=Y2; y3=Y3; y4=Y1+Y2; y5=Y2+Y3; y6=Y3+Y1;
	    z1=(Z0*=2); z2=Z2; z3=Z3; z4=Z1+Z2; z5=Z2+Z3; z6=Z3+Z1;

	    Test(X1*2,Y1*2,Z1*2,x4,y4,z4,x6,y6,z6);
	    Test(x2*2,y2*2,z2*2,x5,y5,z5,x4,y4,z4);
	    Test(x3*2,y3*2,z3*2,x6,y6,z6,x5,y5,z5);

	    if (X0==x1 && Y0==y1 && Z0==z1) {
		X1=x4; Y1=y4; Z1=z4; X2=x5; Y2=y5; Z2=z5; X3=x6; Y3=y6; Z3=z6;
	    }
	}
	x0=(double)X0;
	y0=(double)Y0; rp=sqrt(q=x0*x0+y0*y0); cl=x0/rp; sl=y0/rp;
	z0=(double)Z0; r=sqrt(q+z0*z0); cp=rp/r; sp=z0/r;

	hh=XX*(ssl=sl*sl)+YY*(ccl=cl*cl)-XY*(csl=cl*sl);
	vv=(XX*ccl+YY*ssl+XY*csl)*sp*sp+ZZ*cp*cp-(YZ*sl+ZX*cl)*cp*sp;
	hv=((XX-YY)*csl-(XY/2.0)*(ccl-ssl))*sp+((YZ/2.0)*cl-(ZX/2.0)*sl)*cp;

	a=(hh-vv)/2.0; b=(hh+vv)/2.0; c=sqrt(a*a+hv*hv);

	(void)printf("%lg\t%lg\t%lg\n%lg\t%lg\t%lg\n%lg\t%lg\t%lg\n",
		     x+1.0/2.0,y+1.0/2.0,z+1.0/2.0,
		     atan2(y0,x0)*180.0/M_PI,
		     atan2(z0,rp)*180.0/M_PI,
		     fmod(3.0/4.0*M_PI-atan2(a,hv)/2.0,M_PI)*180.0/M_PI,
		     sqrt(5.0*(b-c)/M),
		     sqrt(5.0*(b+c)/M),
		     sqrt(5.0*L0_Lc/M));
}}

int	main(int argc,char **argv)
{
	switch(argc) {
	    case 2 : OvalFitting(argv[1],1.0,1.0); break;
	    case 3 : OvoidFitting(argv[1],argv[2],1.0,1.0,1.0); break;
	    case 4 : OvalFitting(argv[1],atof(argv[2]),atof(argv[3])); break;
	    case 6 :
OvoidFitting(argv[1],argv[2],atof(argv[3]),atof(argv[4]),atof(argv[5])); break;
	    default: Error("usage : of_float TIFF {Ux Uy}\n"
			   "        of_float directory nameFile {Ux Uy Uz}");
	}
	return 0;
}
