
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rif_f.h"
#include "csi_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

#define LEN	2048

static void	Read(char *path,
		     int *Nx,int *Ny,FOM ***fom,double *base,double *step)
{
	int	bps,n;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	f1,f2;

	if ((bps=ReadImageFile_Float(path,Nx,Ny,fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&f1)!=1 ||
			sscanf(str[n-1],"%lf",&f2)!=1 || f1>=f2)
			u_c=u;
		    else {
			u_c=c; *base=f1; *step=(f2-f1)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",base)!=1 ||
			 sscanf(str[n-1],"%lf",step)!=1 || *step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    *base=0.0; *step=1.0;
	}
}

typedef struct List {
		char		**paths;
		int		h1,v1,d1,h2,v2,d2,x0,y0,z0;
		struct List	*next;
	} List;

int	main(int argc,char **argv)
{
	int	Nx,Ny,Nz,x1,y1,z1,x2,y2,z2,
		i,h1,v1,d1,h2,v2,d2,x0,y0,z0,Nh,Nv,Nd,y,z,x,d,v,h;
	double	PV,base,step;
	char	line[LEN],dir[LEN],NF[LEN],**paths,form[LEN],path[LEN];
	List	*list,*next,**ptr=(&next);
	FOM	**S,**R;

	switch(argc) {
	    case 3 :
		break;
	    case 6 :
		if ((Nx=atoi(argv[1]))<=0 ||
		    (Ny=atoi(argv[2]))<=0 ||
		    (Nz=atoi(argv[3]))<=0) Error("bad image size.");

		x1=y1=z1=0; x2=Nx-1; y2=Ny-1; z2=Nz-1; break;
	    case 9 :
		if ((x1=atoi(argv[1]))>
		    (x2=atoi(argv[4])) ||
		    (y1=atoi(argv[2]))>
		    (y2=atoi(argv[5])) ||
		    (z1=atoi(argv[3]))>
		    (z2=atoi(argv[6]))) Error("bad image area.");

		Nx=x2-x1+1; Ny=y2-y1+1; Nz=z2-z1+1; break;
	    default:
		Error("usage : paste_3d_float PV dir/\n"
		      "        paste_3d_float Nx Ny Nz PV dir/\n"
		      "        paste_3d_float x1 y1 z1 x2 y2 z2 PV dir/");
	}
	PV=atof(argv[argc-2]);

	while (fgets(line,LEN,stdin)!=NULL) {
	    if (line[i=(int)strlen(line)-1]!='\n') {
		(void)fputs("too long line :\n",stderr); Error(line);
	    }
	    line[i]='\0';

	    switch(i=sscanf(line,"%s %s %d %d %d %d %d %d %d %d %d",
				 dir,NF,&h1,&v1,&d1,&h2,&v2,&d2,&x0,&y0,&z0)) {
		case EOF: continue;
		case  2 : h1=v1=d1=x0=y0=z0=0; break;
		case  5 : x0=h1; y0=v1; z0=d1; h1=v1=d1=0; break;
		case  8 : x0=y0=z0=0; break;
		case 11 : break;
		default :
		(void)fputs("bad number of parameters :\n",stderr); Error(line);
	    }
	    paths=CheckSliceImages_Float(dir,NF,&Nh,&Nv,&Nd,NULL);

	    if (i==2 || i==5) {
		h2=Nh-1; v2=Nv-1; d2=Nd-1;
	    }
	    else {
		if (h1<0 || h2>=Nh || v1<0 || v2>=Nv || d1<0 || d2>=Nd) {
		    (void)fputs("bad image area :\n",stderr); Error(line);
		}
		for (d=   0; d<d1; d++) free(paths[d]);
		for (d=d2+1; d<Nd; d++) free(paths[d]);
	    }
	    if ((*ptr=list=(List *)malloc(sizeof(List)))==NULL)
		Error("list memory allocation error.");

	    list->paths=paths; list->h1=h1; list->v1=v1; list->d1=d1;
			       list->h2=h2; list->v2=v2; list->d2=d2;
			       list->x0=x0; list->y0=y0; list->z0=z0;
	    ptr=(&(list->next));
	}
	*ptr=NULL;

	if (argc==3) {
	    if (next==NULL) Error("no image.");

	    list=next; x2=(x1=list->x0)+list->h2-list->h1;
		       y2=(y1=list->y0)+list->v2-list->v1;
		       z2=(z1=list->z0)+list->d2-list->d1;

	    while ((list=list->next)!=NULL) {
		if ((x0=list->x0)<x1) x1=x0;
		if ((y0=list->y0)<y1) y1=y0;
		if ((z0=list->z0)<z1) z1=z0;

		if ((i=x0+list->h2-list->h1)>x2) x2=i;
		if ((i=y0+list->v2-list->v1)>y2) y2=i;
		if ((i=z0+list->d2-list->d1)>z2) z2=i;
	    }
	    Nx=x2-x1+1; Ny=y2-y1+1; Nz=z2-z1+1;
	}
	if ((S=(FOM **)malloc(sizeof(FOM *)*Ny))==NULL ||
	    (*S=(FOM *)malloc(sizeof(FOM)*Ny*Nx))==NULL)
	    Error("image memory allocation error.");

	for (y=1; y<Ny; y++) S[y]=S[y-1]+Nx;

	i=1; for (z=10; z<Nz; z*=10) ++i;
	(void)sprintf(form,"%s/%%0%dd.tif",argv[argc-1],i);

	for (z=0; z<Nz; z++) {
	    for (y=0; y<Ny; y++)
	    for (x=0; x<Nx; x++) S[y][x]=PV;

	    for (list=next; list!=NULL; list=list->next)
		if ((x0=list->x0)<=x2 &&
		    (x=x0+(h2=list->h2)-(h1=list->h1))>=x1 &&
		    (y0=list->y0)<=y2 &&
		    (y=y0+(v2=list->v2)-(v1=list->v1))>=y1 &&
		    (d=z+z1-list->z0)>=0 &&
		    (d+=list->d1)<=list->d2) {
		    Read(list->paths[d],&Nh,&Nv,&R,&base,&step);

		    if (Nh<=h2 || Nv<=v2)
			(void)fprintf(stderr,
				      "%s : bad image size (warning).\n",
				      list->paths[d]);

		    if (x>x2) h2=h1+x2-x0; if (h2>=Nh) h2=Nh-1;
		    if (y>y2) v2=v1+y2-y0; if (v2>=Nv) v2=Nv-1;

		    if (x0>=x1)
			x0-=x1;
		    else {
			h1+=(x1-x0); x0=0;
		    }
		    if (y0>=y1)
			y0-=y1;
		    else {
			v1+=(y1-y0); y0=0;
		    }
		    for (y=y0, v=v1; v<=v2; v++, y++)
		    for (x=x0, h=h1; h<=h2; h++, x++)
			S[y][x]=base+step*R[v][h];

		    for (v=0; v<Nv; v++) free(R[v]); free(R);
		}

	    (void)sprintf(path,form,z);
	    StoreImageFile_Float(path,Nx,Ny,S,SIF_F_desc);
	}
	free(*S); free(S);

	while ((list=next)!=NULL) {
	    for (d=list->d1; d<=list->d2; d++) free(list->paths[d]);

	    free(list->paths); next=list->next; free(list);
	}
	return 0;
}
