
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rif_f.h"
#include "csi_f.h"
#include "sif_f.h"

#define LEN	2048

static void	Read(char *path,int Ox,int Oy,int Nx,int Ny,FOM **F)
{
	int	bps,n,Ix,Iy,X,Y,y,x;
	FOM	**fom,*f;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	min,max,base,step;

	if ((bps=ReadImageFile_Float(path,&Ix,&Iy,&fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&min)!=1 ||
			sscanf(str[n-1],"%lf",&max)!=1 || min>=max)
			u_c=u;
		    else {
			u_c=c; base=min; step=(max-min)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",&base)!=1 ||
			 sscanf(str[n-1],"%lf",&step)!=1 || step<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",path,u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    base=0.0; step=1.0;
	}
	X=Ox+Nx-1; Y=Oy+Ny-1;

	for (y=Iy-1; y>Y; y--) free(fom[y]);

	for (y=Y; y>=Iy; y--)
	    for (f=F[y-Oy]+(x=X)-Ox; x>=Ox; x--) *(f--)=base;

	for (; y>=Oy; y--) {
	    for (f=F[y-Oy]+(x=X)-Ox; x>=Ix; x--) *(f--)=base;

	    for (; x>=Ox; x--) *(f--)=base+step*fom[y][x];

	    free(fom[y]);
	}
	for (; y>=0; y--) free(fom[y]);

	free(fom);
}

static int	Range(int *Ng,int *Nh,int *D1,int *D2,int *Og,int *Oh)
{
	int	g,h,N,L;

	if ((g=(*Nh)+(*D2))>(*Ng)) g=(*Ng);
	if ((h=(*Ng)-(*D1))>(*Nh)) h=(*Nh);

	if (*D2<0) {
	    *Og=0; *Oh=(-(*D2)); *D1-=(*D2); *D2=0; N=1-(*D1);
	}
	else if (*D1>0) {
	    *Og=(*D1); *Oh=0; *D2-=(*D1); *D1=0; N=(*D2)+1;
	}
	else {
	    *Og=(*Oh)=0; N=(-(*D1)>(*D2))?1-(*D1):*D2+1;
	}
	N+=(((*Ng=g-(*Og))>(*Nh=h-(*Oh)))?*Ng:*Nh);

	for (L=1; L<N; L<<=1) ; return L;
}

typedef struct {
		double	r,i;
	} Complex;

#ifndef	M_PI
#define	M_PI	3.14159265358979323846
#endif

static Complex	*Table(int L)
{
	Complex	*E,*e;
	int	l;
	double	t;

	if ((E=(Complex *)malloc(sizeof(Complex)*L*2))==NULL) return NULL;

	for (e=E, l=0; l<L; l++, e++) {
	    t=M_PI*(double)l/(double)L; e[L].r=(-(e->r=cos(t)));
					e[L].i=(-(e->i=sin(t)));
	}
	return E;
}

static void	FFT(int S,int L,Complex *E,Complex *C)
{
	int	j,k,m,n,p,q,L2=L>>1;
	Complex	c,*e,*a,*b;
	double	r,i,d,s=(double)S;

	for (j=L-1, k=L-2; k>0; k--) {
	    for (m=L2; (j^=m)&m; m>>=1) ;

	    if (j>k) {
		c=C[j]; C[j]=C[k]; C[k]=c;
	    }
	}
	for (j=2, k=1, m=L2, n=L; n>1; n=m, m>>=1, k=j, j<<=1)
	    for (e=E, p=0; p<k; p++, e+=n) {
		r=e->r; i=e->i*s;
		for (a=C+p, q=0; q<m; q++, a+=j) {
		    c=(*(b=a+k)); b->r=a->r-(d=c.r*r-c.i*i); a->r+=d;
				  b->i=a->i-(d=c.r*i+c.i*r); a->i+=d;
		}
	    }
}

static Complex	Product(double r1,double i1,double r2,double i2)
{
	Complex	c;

	c.r=r1*r2-i1*i2; c.i=r1*i2+i1*r2; return c;
}

double	*RMSD(char **Pg,int Ngx,int Ngy,int Ngz,
	      char **Ph,int Nhx,int Nhy,int Nhz,
	      int Dx1,int Dy1,int Dz1,int Dx2,int Dy2,int Dz2)
{
	int	Lx,Ogx,Ohx,Ly,Ogy,Ohy,Lz,Ogz,Ohz,x,y,z,
		gx,gy,gz,hx,hy,hz,Lxx,Lyy,Nghx,Nghy,Nghz,
		Lx1,Ly1,Lz1,Lx2,Ly2,Lz2,Ngx1,Ngy1,Ngz1,Nhx1,Nhy1,Nhz1;
	double	*P,r,i,t,d,rg,ig,rh,ih,*p,l,lz,lzy,lzyx,
		ngx,ngy,ngz,ngxy,ngxz,ngx2,ngy2,ngz2,ngx2y,ngxy2,ngx2y2,
		nhx,nhy,nhz,nhxy,nhxz,nhx2,nhy2,nhz2,nhx2y,nhxy2,nhx2y2;
	FOM	**F,*f;
	Complex	*Ex,*Ey,*Ez,***Q,*U,*V,
		***Qz,**Qy,*Qx,*u,*v,*ex,*ey,*ez,g,h,c,cg,ch;

	Lx=Range(&Ngx,&Nhx,&Dx1,&Dx2,&Ogx,&Ohx);
	Ly=Range(&Ngy,&Nhy,&Dy1,&Dy2,&Ogy,&Ohy);
	Lz=Range(&Ngz,&Nhz,&Dz1,&Dz2,&Ogz,&Ohz);

	x=(Ngx>Nhx)?Ngx:Nhx;
	y=(Ngy>Nhy)?Ngy:Nhy;
	z=(Ly>Lz)?Ly:Lz;

	if ((P=(double *)malloc(sizeof(double)*(Dz2-Dz1+1)
					      *(Dy2-Dy1+1)
					      *(Dx2-Dx1+1)))==NULL ||
	    (F=(FOM **)malloc(sizeof(FOM *)*y))==NULL ||
	    (*F=(FOM *)malloc(sizeof(FOM)*y*x))==NULL ||
	    (Ex=Table(Lx))==NULL ||
	    (Ey=Table(Ly))==NULL ||
	    (Ez=Table(Lz))==NULL ||
	    (Q=(Complex ***)malloc(sizeof(Complex **)*Lz))==NULL ||
	    (*Q=(Complex **)malloc(sizeof(Complex *)*Lz*Ly))==NULL ||
	    (**Q=(Complex *)malloc(sizeof(Complex)*Lz*Ly*Lx))==NULL ||
	    (U=(Complex *)malloc(sizeof(Complex)*z))==NULL ||
	    (V=(Complex *)malloc(sizeof(Complex)*z))==NULL) return NULL;

	for (z=1; z<y; z++) F[z]=F[z-1]+x;

	for (y=1; y<Ly; y++) Q[0][y]=Q[0][y-1]+Lx;
	for (z=1; z<Lz; z++) {
	    Q[z]=Q[z-1]+Ly; for (y=0; y<Ly; y++) Q[z][y]=Q[z][y-1]+Lx;
	}
	Lx1=Lx-1; Ly1=Ly-1; Lz1=Lz-1;
	Lx2=Lx*2; Ly2=Ly*2; Lz2=Lz*2;

	Ngx1=Ngx-1; Ngy1=Ngy-1; Ngz1=Ngz-1;
	Nhx1=Nhx-1; Nhy1=Nhy-1; Nhz1=Nhz-1;

	ngx=(double)Ngx; ngy=(double)Ngy; ngz=(double)Ngz;
	ngxy=ngx*ngy; ngxz=ngx*ngz;
	ngx2=(double)(Ngx%2); ngy2=(double)(Ngy%2); ngz2=(double)(Ngz%2);
	ngx2y=ngx2*ngy; ngxy2=ngx*ngy2; ngx2y2=ngx2*ngy2;

	nhx=(double)Nhx; nhy=(double)Nhy; nhz=(double)Nhz;
	nhxy=nhx*nhy; nhxz=nhx*nhz;
	nhx2=(double)(Nhx%2); nhy2=(double)(Nhy%2); nhz2=(double)(Nhz%2);
	nhx2y=nhx2*nhy; nhxy2=nhx*nhy2; nhx2y2=nhx2*nhy2;
/*
    ----
*/
	for (Qz=Q, z=0; z<Lz; z++, Qz++) {
	    if (z<Ngz) {
		Read(Pg[Ogz+z],Ogx,Ogy,Ngx,Ngy,F);

		Qy=(*Qz);
		for (y=0; y<Ly; y++) {
		    Qx=(*(Qy++)); x=0;
		    if (y<Ngy)
			for (f=F[y]; x<Ngx; x++) {
			    r=(*(f++)); (Qx++)->r=r*r;
			}

		    for (; x<Lx; x++) (Qx++)->r=0.0;
		}
	    }
	    else
		for (Qy=(*Qz), y=0; y<Ly; y++, Qy++)
		for (Qx=(*Qy), x=0; x<Lx; x++, Qx++) Qx->r=0.0;

	    if (z<Nhz) {
		Read(Ph[Ohz+z],Ohx,Ohy,Nhx,Nhy,F);

		Qy=(*Qz);
		for (y=0; y<Ly; y++) {
		    Qx=(*(Qy++)); x=0;
		    if (y<Nhy)
			for (f=F[y]; x<Nhx; x++) {
			    i=(*(f++)); (Qx++)->i=i*i;
			}

		    for (; x<Lx; x++) (Qx++)->i=0.0;
		}
	    }
	    else
		for (Qy=(*Qz), y=0; y<Ly; y++, Qy++)
		for (Qx=(*Qy), x=0; x<Lx; x++, Qx++) Qx->i=0.0;

	    if (z<Ngz || z<Nhz) {
		Qy=(*Qz); for (y=0; y<Ngy || y<Nhy; y++) FFT(-1,Lx,Ex,*(Qy++));

		Qy=(*Qz);
		for (x=0; x<Lx; x++) {
		    for (u=U, y=0; y<Ly; y++) *(u++)=Qy[y][x];
		    FFT(-1,Ly,Ey,U);
		    for (u=U, y=0; y<Ly; y++) Qy[y][x]=(*(u++));
		}
	    }
	}
/*
    y=0 & x=0
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][0][0]; FFT(-1,Lz,Ez,U);

	U->r=U->r*nhxy*nhz+ngxy*ngz*U->i; U->i=0.0;
	for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=U+Lz1;
	     u<v;
	     ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--) {
	    t=ez->r/ez->i;
	    r=Ez[hz].r; i=Ez[hz].i; d=nhxy*(r+i*t);
	    g=Product(u->r+v->r,u->i-v->i,d*r, d*i);
	    r=Ez[gz].r; i=Ez[gz].i; d=ngxy*(r+i*t);
	    h=Product(d*r,-d*i,u->i+v->i,u->r-v->r);
	    v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	}
	u->r=u->r*nhxy*nhz2+ngxy*ngz2*u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][0][0]=(*(u++));
/*
    y=0 & x=1~Lx/2-1
*/
	for (ex=Ex+1, gx=Ngx1, hx=Nhx1, x=1, Lxx=Lx1;
	     x<Lxx;
	     ex++, gx=(gx+Ngx1)%Lx2, hx=(hx+Nhx1)%Lx2, x++, Lxx--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][0][x]; *(v++)=Q[z][0][Lxx];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    t=ex->r/ex->i;
	    r=Ex[gx].r; i=Ex[gx].i; d=ngy*(r+i*t); rg=d*r; ig=d*i;
	    r=Ex[hx].r; i=Ex[hx].i; d=nhy*(r+i*t); rh=d*r; ih=d*i;
	    g=Product(U->r+V->r,U->i-V->i,rh*nhz, ih*nhz);
	    h=Product(rg*ngz,-ig*ngz,U->i+V->i,U->r-V->r);
	    V->r=U->r=(g.r+h.r)/2.0; V->i=(-(U->i=(g.i+h.i)/2.0));
	    for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=V+Lz1, z=1;
		 z<Lz;
		 ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--, z++) {
		t=ez->r/ez->i;
		r=Ez[hz].r; i=Ez[hz].i; d=r+i*t; c=Product(rh,ih,d*r,d*i);
		g=Product(u->r+v->r,u->i-v->i,c.r, c.i);
		r=Ez[gz].r; i=Ez[gz].i; d=r+i*t; c=Product(rg,ig,d*r,d*i);
		h=Product(c.r,-c.i,u->i+v->i,u->r-v->r);
		v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][0][x]=(*(u++)); Q[z][0][Lxx]=(*(v++));
	    }
	}
/*
    y=0 & x=Lx/2
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][0][x]; FFT(-1,Lz,Ez,U);

	U->r=U->r*nhx2y*nhz+ngx2y*ngz*U->i; U->i=0.0;
	for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=U+Lz1;
	     u<v;
	     ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--) {
	    t=ez->r/ez->i;
	    r=Ez[hz].r; i=Ez[hz].i; d=nhx2y*(r+i*t);
	    g=Product(u->r+v->r,u->i-v->i,d*r, d*i);
	    r=Ez[gz].r; i=Ez[gz].i; d=ngx2y*(r+i*t);
	    h=Product(d*r,-d*i,u->i+v->i,u->r-v->r);
	    v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	}
	u->r=u->r*nhx2y*nhz2+ngx2y*ngz2*u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][0][x]=(*(u++));
/*
    y=1~Ly/2-1 & x=0
*/
	for (ey=Ey+1, gy=Ngy1, hy=Nhy1, y=1, Lyy=Ly1;
	     y<Lyy;
	     ey++, gy=(gy+Ngy1)%Ly2, hy=(hy+Nhy1)%Ly2, y++, Lyy--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][y][0]; *(v++)=Q[z][Lyy][0];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    t=ey->r/ey->i;
	    r=Ey[gy].r; i=Ey[gy].i; d=r+i*t; rg=d*r; ig=d*i;
	    r=Ey[hy].r; i=Ey[hy].i; d=r+i*t; rh=d*r; ih=d*i;
	    g=Product(U->r+V->r,U->i-V->i,rh*nhxz, ih*nhxz);
	    h=Product(rg*ngxz,-ig*ngxz,U->i+V->i,U->r-V->r);
	    V->r=U->r=(g.r+h.r)/2.0; V->i=(-(U->i=(g.i+h.i)/2.0));
	    for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=V+Lz1, z=1;
		 z<Lz;
		 ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--, z++) {
		t=ez->r/ez->i;
		r=Ez[hz].r; i=Ez[hz].i; d=nhx*(r+i*t); c=Product(rh,ih,d*r,d*i);
		g=Product(u->r+v->r,u->i-v->i,c.r, c.i);
		r=Ez[gz].r; i=Ez[gz].i; d=ngx*(r+i*t); c=Product(rg,ig,d*r,d*i);
		h=Product(c.r,-c.i,u->i+v->i,u->r-v->r);
		v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][y][0]=(*(u++)); Q[z][Lyy][0]=(*(v++));
	    }
/*
    y=1~Ly/2-1 & x=1~Lx-1
*/
	    for (ex=Ex+1, gx=Ngx1, hx=Nhx1, x=1, Lxx=Lx1;
		 x<Lx;
		 ex++, gx=(gx+Ngx1)%Lx2, hx=(hx+Nhx1)%Lx2, x++, Lxx--) {
		for (u=U, v=V, z=0; z<Lz; z++) {
		    *(u++)=Q[z][y][x]; *(v++)=Q[z][Lyy][Lxx];
		}
		FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

		t=ex->r/ex->i;
		r=Ex[gx].r; i=Ex[gx].i; d=r+i*t; cg=Product(rg,ig,d*r,d*i);
		r=Ex[hx].r; i=Ex[hx].i; d=r+i*t; ch=Product(rh,ih,d*r,d*i);
		g=Product(U->r+V->r,U->i-V->i,ch.r*nhz, ch.i*nhz);
		h=Product(cg.r*ngz,-cg.i*ngz,U->i+V->i,U->r-V->r);
		V->r=U->r=(g.r+h.r)/2.0; V->i=(-(U->i=(g.i+h.i)/2.0));
		for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=V+Lz1, z=1;
		     z<Lz;
		     ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--, z++) {
		    t=ez->r/ez->i;
		    r=Ez[hz].r; i=Ez[hz].i;
		    d=r+i*t; c=Product(ch.r,ch.i,d*r,d*i);
		    g=Product(u->r+v->r,u->i-v->i,c.r, c.i);
		    r=Ez[gz].r; i=Ez[gz].i;
		    d=r+i*t; c=Product(cg.r,cg.i,d*r,d*i);
		    h=Product(c.r,-c.i,u->i+v->i,u->r-v->r);
		    v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
		}
		FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
		for (u=U, v=V, z=0; z<Lz; z++) {
		    Q[z][y][x]=(*(u++)); Q[z][Lyy][Lxx]=(*(v++));
		}
	    }
	}
/*
    y=Ly/2 & x=0
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][y][0]; FFT(-1,Lz,Ez,U);

	U->r=U->r*nhxy2*nhz+ngxy2*ngz*U->i; U->i=0.0;
	for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=U+Lz1;
	     u<v;
	     ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--) {
	    t=ez->r/ez->i;
	    r=Ez[hz].r; i=Ez[hz].i; d=nhxy2*(r+i*t);
	    g=Product(u->r+v->r,u->i-v->i,d*r, d*i);
	    r=Ez[gz].r; i=Ez[gz].i; d=ngxy2*(r+i*t);
	    h=Product(d*r,-d*i,u->i+v->i,u->r-v->r);
	    v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	}
	u->r=u->r*nhxy2*nhz2+ngxy2*ngz2*u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][y][0]=(*(u++));
/*
    y=Ly/2 & x=1~Lx/2-1
*/
	for (ex=Ex+1, gx=Ngx1, hx=Nhx1, x=1, Lxx=Lx1;
	     x<Lxx;
	     ex++, gx=(gx+Ngx1)%Lx2, hx=(hx+Nhx1)%Lx2, x++, Lxx--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][y][x]; *(v++)=Q[z][y][Lxx];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    t=ex->r/ex->i;
	    r=Ex[gx].r; i=Ex[gx].i; d=ngy2*(r+i*t); rg=d*r; ig=d*i;
	    r=Ex[hx].r; i=Ex[hx].i; d=nhy2*(r+i*t); rh=d*r; ih=d*i;
	    g=Product(U->r+V->r,U->i-V->i,rh*nhz, ih*nhz);
	    h=Product(rg*ngz,-ig*ngz,U->i+V->i,U->r-V->r);
	    V->r=U->r=(g.r+h.r)/2.0; V->i=(-(U->i=(g.i+h.i)/2.0));
	    for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=V+Lz1, z=1;
		 z<Lz;
		 ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--, z++) {
		t=ez->r/ez->i;
		r=Ez[hz].r; i=Ez[hz].i; d=r+i*t; c=Product(rh,ih,d*r,d*i);
		g=Product(u->r+v->r,u->i-v->i,c.r, c.i);
		r=Ez[gz].r; i=Ez[gz].i; d=r+i*t; c=Product(rg,ig,d*r,d*i);
		h=Product(c.r,-c.i,u->i+v->i,u->r-v->r);
		v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][y][x]=(*(u++)); Q[z][y][Lxx]=(*(v++));
	    }
	}
/*
    y=Ly/2 & x=Lx/2
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][y][x]; FFT(-1,Lz,Ez,U);

	U->r=U->r*nhx2y2*nhz+ngx2y2*ngz*U->i; U->i=0.0;
	for (ez=Ez+1, gz=Ngz1, hz=Nhz1, u=U+1, v=U+Lz1;
	     u<v;
	     ez++, gz=(gz+Ngz1)%Lz2, hz=(hz+Nhz1)%Lz2, u++, v--) {
	    t=ez->r/ez->i;
	    r=Ez[hz].r; i=Ez[hz].i; d=nhx2y2*(r+i*t);
	    g=Product(u->r+v->r,u->i-v->i,d*r, d*i);
	    r=Ez[gz].r; i=Ez[gz].i; d=ngx2y2*(r+i*t);
	    h=Product(d*r,-d*i,u->i+v->i,u->r-v->r);
	    v->r=u->r=(g.r+h.r)/2.0; v->i=(-(u->i=(g.i+h.i)/2.0));
	}
	u->r=u->r*nhx2y2*nhz2+ngx2y2*ngz2*u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][y][x]=(*(u++));
/*
    ----
*/
	p=P;
	for (z=Dz1; z<=Dz2; z++) {
	    Qy=Q[(z<0)?Lz+z:z];
	    for (x=0; x<Lx; x++) {
		for (u=U, y=0; y<Ly; y++) *(u++)=Qy[y][x];
		FFT(1,Ly,Ey,U);
		for (u=U, y=0; y<Ly; y++) Qy[y][x]=(*(u++));
	    }
	    for (y=Dy1; y<=Dy2; y++) {
		FFT(1,Lx,Ex,Qx=Qy[(y<0)?Ly+y:y]);

		for (x=Dx1; x<=Dx2; x++) *(p++)=Qx[(x<0)?Lx+x:x].r;
	    }
	}
/*
    ----
*/
	for (Qz=Q, z=0; z<Lz; z++, Qz++) {
	    if (z<Ngz) {
		Read(Pg[Ogz+z],Ogx,Ogy,Ngx,Ngy,F);

		Qy=(*Qz);
		for (y=0; y<Ly; y++) {
		    Qx=(*(Qy++)); x=0;
		    if (y<Ngy)
			for (f=F[y]; x<Ngx; x++) (Qx++)->r=(*(f++));

		    for (; x<Lx; x++) (Qx++)->r=0.0;
		}
	    }
	    else
		for (Qy=(*Qz), y=0; y<Ly; y++, Qy++)
		for (Qx=(*Qy), x=0; x<Lx; x++, Qx++) Qx->r=0.0;

	    if (z<Nhz) {
		Read(Ph[Ohz+z],Ohx,Ohy,Nhx,Nhy,F);

		Qy=(*Qz);
		for (y=0; y<Ly; y++) {
		    Qx=(*(Qy++)); x=0;
		    if (y<Nhy)
			for (f=F[y]; x<Nhx; x++) (Qx++)->i=(*(f++));

		    for (; x<Lx; x++) (Qx++)->i=0.0;
		}
	    }
	    else
		for (Qy=(*Qz), y=0; y<Ly; y++, Qy++)
		for (Qx=(*Qy), x=0; x<Lx; x++, Qx++) Qx->i=0.0;

	    if (z<Ngz || z<Nhz) {
		Qy=(*Qz); for (y=0; y<Ngy || y<Nhy; y++) FFT(-1,Lx,Ex,*(Qy++));

		Qy=(*Qz);
		for (x=0; x<Lx; x++) {
		    for (u=U, y=0; y<Ly; y++) *(u++)=Qy[y][x];
		    FFT(-1,Ly,Ey,U);
		    for (u=U, y=0; y<Ly; y++) Qy[y][x]=(*(u++));
		}
	    }
	}
/*
    y=0 & x=0
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][0][0]; FFT(-1,Lz,Ez,U);

	U->r*=U->i; U->i=0.0;
	for (u=U+1, v=U+Lz1; u<v; u++, v--) {
	    *u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
		       (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							 v->i=(-u->i);
	}
	u->r*=u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][0][0]=(*(u++));
/*
    y=0 & x=1~Lx/2-1
*/
	for (x=1, Lxx=Lx1; x<Lxx; x++, Lxx--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][0][x]; *(v++)=Q[z][0][Lxx];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    *U=Product((U->r+V->r)/2.0,(U->i-V->i)/2.0,
		       (U->i+V->i)/2.0,(U->r-V->r)/2.0); V->r=  U->r;
							 V->i=(-U->i);
	    for (u=U+1, v=V+Lz1, z=1; z<Lz; z++, u++, v--) {
		*u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
			   (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							     v->i=(-u->i);
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][0][x]=(*(u++)); Q[z][0][Lxx]=(*(v++));
	    }
	}
/*
    y=0 & x=Lx/2
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][0][x]; FFT(-1,Lz,Ez,U);

	U->r*=U->i; U->i=0.0;
	for (u=U+1, v=U+Lz1; u<v; u++, v--) {
	    *u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
		       (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							 v->i=(-u->i);
	}
	u->r*=u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][0][x]=(*(u++));
/*
    y=1~Ly/2-1 & x=0
*/
	for (y=1, Lyy=Ly1; y<Lyy; y++, Lyy--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][y][0]; *(v++)=Q[z][Lyy][0];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    *U=Product((U->r+V->r)/2.0,(U->i-V->i)/2.0,
		       (U->i+V->i)/2.0,(U->r-V->r)/2.0); V->r=  U->r;
							 V->i=(-U->i);
	    for (u=U+1, v=V+Lz1, z=1; z<Lz; z++, u++, v--) {
		*u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
			   (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							     v->i=(-u->i);
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][y][0]=(*(u++)); Q[z][Lyy][0]=(*(v++));
	    }
/*
    y=1~Ly/2-1 & x=1~Lx-1
*/
	    for (x=1, Lxx=Lx1; x<Lx; x++, Lxx--) {
		for (u=U, v=V, z=0; z<Lz; z++) {
		    *(u++)=Q[z][y][x]; *(v++)=Q[z][Lyy][Lxx];
		}
		FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

		*U=Product((U->r+V->r)/2.0,(U->i-V->i)/2.0,
			   (U->i+V->i)/2.0,(U->r-V->r)/2.0); V->r=  U->r;
							     V->i=(-U->i);
		for (u=U+1, v=V+Lz1, z=1; z<Lz; z++, u++, v--) {
		    *u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
			       (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
								 v->i=(-u->i);
		}
		FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
		for (u=U, v=V, z=0; z<Lz; z++) {
		    Q[z][y][x]=(*(u++)); Q[z][Lyy][Lxx]=(*(v++));
		}
	    }
	}
/*
    y=Ly/2 & x=0
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][y][0]; FFT(-1,Lz,Ez,U);

	U->r*=U->i; U->i=0.0;
	for (u=U+1, v=U+Lz1; u<v; u++, v--) {
	    *u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
		       (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							 v->i=(-u->i);
	}
	u->r*=u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][y][0]=(*(u++));
/*
    y=Ly/2 & x=1~Lx/2-1
*/
	for (x=1, Lxx=Lx1; x<Lxx; x++, Lxx--) {
	    for (u=U, v=V, z=0; z<Lz; z++) {
		*(u++)=Q[z][y][x]; *(v++)=Q[z][y][Lxx];
	    }
	    FFT(-1,Lz,Ez,U); FFT(-1,Lz,Ez,V);

	    *U=Product((U->r+V->r)/2.0,(U->i-V->i)/2.0,
		       (U->i+V->i)/2.0,(U->r-V->r)/2.0); V->r=  U->r;
							 V->i=(-U->i);
	    for (u=U+1, v=V+Lz1, z=1; z<Lz; z++, u++, v--) {
		*u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
			   (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							     v->i=(-u->i);
	    }
	    FFT(1,Lz,Ez,U); FFT(1,Lz,Ez,V);
	    for (u=U, v=V, z=0; z<Lz; z++) {
		Q[z][y][x]=(*(u++)); Q[z][y][Lxx]=(*(v++));
	    }
	}
/*
    y=Ly/2 & x=Lx/2
*/
	for (u=U, z=0; z<Lz; z++) *(u++)=Q[z][y][x]; FFT(-1,Lz,Ez,U);

	U->r*=U->i; U->i=0.0;
	for (u=U+1, v=U+Lz1; u<v; u++, v--) {
	    *u=Product((u->r+v->r)/2.0,(u->i-v->i)/2.0,
		       (u->i+v->i)/2.0,(u->r-v->r)/2.0); v->r=  u->r ;
							 v->i=(-u->i);
	}
	u->r*=u->i; u->i=0.0;

	FFT(1,Lz,Ez,U); for (u=U, z=0; z<Lz; z++) Q[z][y][x]=(*(u++));
/*
    ----
*/
	l=(double)Lx*(double)Ly*(double)Lz; Nghx=Ngx-Nhx;
					    Nghy=Ngy-Nhy;
					    Nghz=Ngz-Nhz; p=P;
	for (z=Dz1; z<=Dz2; z++) {
	    if (z<0) {
		Qy=Q[Lz+z]; lz=l*(double)((z>=Nghz)?Ngz:Nhz+z);
	    }
	    else {
		Qy=Q[   z]; lz=l*(double)((z>=Nghz)?Ngz-z:Nhz);
	    }
	    for (x=0; x<Lx; x++) {
		for (u=U, y=0; y<Ly; y++) *(u++)=Qy[y][x];
		FFT(1,Ly,Ey,U);
		for (u=U, y=0; y<Ly; y++) Qy[y][x]=(*(u++));
	    }
	    for (y=Dy1; y<=Dy2; y++) {
		if (y<0) {
		    Qx=Qy[Ly+y]; lzy=lz*(double)((y>=Nghy)?Ngy:Nhy+y);
		}
		else {
		    Qx=Qy[   y]; lzy=lz*(double)((y>=Nghy)?Ngy-y:Nhy);
		}
		FFT(1,Lx,Ex,Qx);

		for (x=Dx1; x<=Dx2; x++) {
		    if (x<0) {
		d=(*p)-2.0*Qx[Lx+x].r; lzyx=lzy*(double)((x>=Nghx)?Ngx:Nhx+x);
		    }
		    else {
		d=(*p)-2.0*Qx[   x].r; lzyx=lzy*(double)((x>=Nghx)?Ngx-x:Nhx);
		    }
		    *(p++)=(d<0.0)?0.0:sqrt(d/lzyx);
		}
	    }
	}
	free(V); free(U); free(**Q); free(*Q); free(Q);
	free(Ez); free(Ey); free(Ex); free(*F); free(F); return P;
}

int	main(int argc,char **argv)
{
	char	**Pg,**Ph,form[LEN],path[LEN];
	int	Ngx,Ngy,Ngz,Nhx,Nhy,Nhz,Dx1,Dy1,Dz1,Dx2,Dy2,Dz2,x,y,X,Y,Z,z,l;
	double	*rmsd,D,*d;
	FOM	**fom,*f;

	if (argc==3 || argc==4 || argc==7 || argc==8) {
	    (void)ReadImageFile_Float(*(Pg=argv+1),&Ngx,&Ngy,NULL,NULL); Ngz=1;
	    (void)ReadImageFile_Float(*(Ph=argv+2),&Nhx,&Nhy,NULL,NULL); Nhz=1;

	    if (argc==3 || argc==4) {
		Dx1=1-Nhx; Dy1=1-Nhy;
		Dx2=Ngx-1; Dy2=Ngy-1;
	    }
	    else {
		Dx1=atoi(argv[3]); Dy1=atoi(argv[4]);
		Dx2=atoi(argv[5]); Dy2=atoi(argv[6]);
	    }
	    Dz1=Dz2=0;
	}
	else if (argc==5 || argc==6 || argc==11 || argc==12) {
	    Pg=CheckSliceImages_Float(argv[1],argv[2],&Ngx,&Ngy,&Ngz,NULL);
	    Ph=CheckSliceImages_Float(argv[3],argv[4],&Nhx,&Nhy,&Nhz,NULL);

	    if (argc==5 || argc==6) {
		Dx1=1-Nhx; Dy1=1-Nhy; Dz1=1-Nhz;
		Dx2=Ngx-1; Dy2=Ngy-1; Dz2=Ngz-1;
	    }
	    else {
		Dx1=atoi(argv[5]); Dy1=atoi(argv[6]); Dz1=atoi(argv[ 7]);
		Dx2=atoi(argv[8]); Dy2=atoi(argv[9]); Dz2=atoi(argv[10]);
	    }
	}
	else {
	    (void)fputs(
"usage : rmsd_float TIFF1 TIFF2 {Dx1 Dy1 Dx2 Dy2} {RMSD.tif}\n"
"        rmsd_float dir1/ NF1 dir2/ NF2 {Dx1 Dy1 Dz1 Dx2 Dy2 Dz2} {RMSD/}\n",
			stderr); exit(1);
	}
	if (Dx1<=(-Nhx) || Dx2>=Ngx || Dx1>Dx2 ||
	    Dy1<=(-Nhy) || Dy2>=Ngy || Dy1>Dy2 ||
	    Dz1<=(-Nhz) || Dz2>=Ngz || Dz1>Dz2) {
	    (void)fputs("bad range of displacement.\n",stderr); exit(1);
	}
	if ((rmsd=RMSD(Pg,Ngx,Ngy,Ngz,
		       Ph,Nhx,Nhy,Nhz,
		       Dx1,Dy1,Dz1,Dx2,Dy2,Dz2))==NULL ||
	    ((argc==4 || argc==8 || argc==6 || argc==12) &&
	     ((fom=(FOM **)malloc(sizeof(FOM *)*(Dy2-Dy1+1)))==NULL ||
	      (*fom=(FOM *)malloc(sizeof(FOM)*(Dy2-Dy1+1)
					     *(Dx2-Dx1+1)))==NULL))) {
	    (void)fputs("memory allocation error.\n",stderr); exit(1);
	}
	if (argc==4 || argc==8 || argc==6 || argc==12) {
	    x=Dx2-Dx1+1; for (y=1; y<Dy2-Dy1+1; y++) fom[y]=fom[y-1]+x;
	}
	if (argc==3 || argc==4 || argc==7 || argc==8) {
	    X=Dx1; Y=Dy1; D=(*rmsd);
	    for (d=rmsd, y=Dy1; y<=Dy2; y++)
	    for (        x=Dx1; x<=Dx2; x++, d++)
		if (*d<D) {
		    X=x; Y=y; D=(*d);
		}

	    (void)printf("%d\t%d\t%le\n",X,Y,D);
#ifdef	SPR
	    if (X>Dx1 && X<Dx2 && Y>Dy1 && Y<Dy2)
{
	size_t	i=x-Dx1,j=i*(Y-Dy1)+X-Dx1;
	double  w=rmsd[j-1],e=rmsd[j+1],we=w-e,weD=we/(w+e-2.0*D)/2.0,
		s=rmsd[j-i],n=rmsd[j+i],sn=s-n,snD=sn/(s+n-2.0*D)/2.0;

	(void)printf("%lf\t%lf\t%le\n",(double)X+weD,
				       (double)Y+snD,
				       D-(we*weD+sn*snD)/4.0);
}
#endif
	    if (argc==4 || argc==8) {
		d=rmsd; f=(*fom);
		for (y=Dy1; y<=Dy2; y++)
		for (x=Dx1; x<=Dx2; x++) *(f++)=(*(d++));

		StoreImageFile_Float(argv[argc-1],x-Dx1,y-Dy1,fom,SIF_F_desc);
	    }
	}
	else {
	    X=Dx1; Y=Dy1; Z=Dz1; D=(*rmsd);
	    for (d=rmsd, z=Dz1; z<=Dz2; z++)
	    for (        y=Dy1; y<=Dy2; y++)
	    for (        x=Dx1; x<=Dx2; x++, d++)
		if (*d<D) {
		    X=x; Y=y; Z=z; D=(*d);
		}

	    (void)printf("%d\t%d\t%d\t%le\n",X,Y,Z,D);
#ifdef	SPR
	    if (X>Dx1 && X<Dx2 && Y>Dy1 && Y<Dy2 && Z>Dz1 && Z<Dz2)
{
	size_t	i=x-Dx1,j=i*(y-Dy1),k=j*(Z-Dz1)+i*(Y-Dy1)+X-Dx1;
	double  w=rmsd[k-1],e=rmsd[k+1],we=w-e,weD=we/(w+e-2.0*D)/2.0,
		s=rmsd[k-i],n=rmsd[k+i],sn=s-n,snD=sn/(s+n-2.0*D)/2.0,
		b=rmsd[k-j],a=rmsd[k+j],ba=b-a,baD=ba/(b+a-2.0*D)/2.0;

	(void)printf("%lf\t%lf\t%lf\t%le\n",(double)X+weD,
					    (double)Y+snD,
					    (double)Z+baD,
					    D-(we*weD+sn*snD+ba*baD)/4.0);
}
#endif
	    if (argc==6 || argc==12) {
		l=1; for (z=10; z<Dz2-Dz1+1; z*=10) ++l;
		(void)sprintf(form,"%s/%%0%dd.tif",argv[argc-1],l);

		d=rmsd;
		for (z=Dz1; z<=Dz2; z++) {
		    f=(*fom);
		    for (y=Dy1; y<=Dy2; y++)
		    for (x=Dx1; x<=Dx2; x++) *(f++)=(*(d++));

		    (void)sprintf(path,form,z-Dz1);
		    StoreImageFile_Float(path,x-Dx1,y-Dy1,fom,SIF_F_desc);
		}
	    }
	    for (z=0; z<Nhz; z++) free(Ph[z]); free(Ph);
	    for (z=0; z<Ngz; z++) free(Pg[z]); free(Pg);
	}
	if (argc==4 || argc==8 || argc==6 || argc==12) {
	    free(*fom); free(fom);
	}
	free(rmsd);

	return 0;
}
