
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define ERROR(msg)	(void)fputs(msg,stderr)

static void	Error(msg)
char		*msg;
{
	ERROR(msg); (void)fputc('\n',stderr); exit(1);
}

#include "tio.h"

#define STR_LEN	1024

int	main(argc,argv)
int	argc;
char	**argv;
{
	size_t	len=0;
	char	*ID,str[STR_LEN],MN[4],
		*NAMFS="no allocatable memory for string.",
		*NOBSPP="no or bad samples per pixel.";
	Word4	IW,IL,CMC,CMBC,RPS,SPI,ofs,CMO,idx;
	Word2	PI,PC,SPP,BPS,IC,DP,FO,tags=10;
	Byte	*CMB;

	if (argc!=3 && argc!=4) Error("usage : sid orgTIFF {string} newTIFF");

	if (argc==4) {
	    if (argv[2][0]!='-' || argv[2][1]!='\0')
		len=strlen(ID=argv[2])+1;
	    else {
		len=1; ID=NULL;
		while (fgets(str,STR_LEN,stdin)!=NULL) {
		    len+=strlen(str);
		    if (ID!=NULL) {
			if ((ID=(char *)realloc(ID,len))==NULL) Error(NAMFS);

			(void)strcat(ID,str);
		    }
		    else
			if ((ID=strdup(str))==NULL) Error(NAMFS);
		}
		if (len>1 && ID[len-2]=='\n') ID[--len-1]='\0';
	    }
	    if (len>1) ++tags;
	}
	if ((file=fopen(argv[1],"rb"))==NULL) Error("file not found.");

	Nread((Byte *)MN,(size_t)4);
	if (memcmp(MN,"II*\0",(size_t)4) &&
	    memcmp(MN,"MM\0*",(size_t)4)) Error("bad magic number.");

	if (*(Word2 *)(MN+2)=='*') {
	    Read=Nread; Write=Nwrite;
	}
	else {
	    Read=Rread; Write=Rwrite;
	}
	if ((IFD=Get4())==0) Error("no image file directory.");

	if ((IW=GetIFD4(0x100,0))==0) NoB_Error("image width");
	if ((IL=GetIFD4(0x101,0))==0) NoB_Error("image length");
	if ((PI=GetIFD3(0x106,5))>=5) NoB_Error("photometric interpretation");
	if ((PC=GetIFD3(0x11c,1))!=1) NoB_Error("planar configuration");

	if (PI!=2) {
	    if ((SPP=GetIFD3(0x115,1))!=1) Error(NOBSPP);

	    BPS=GetIFD3(0x102,1);
	}
	else {
	    if ((SPP=GetIFD3(0x115,1))!=3) Error(NOBSPP);

	    if (GetIFD(0x102)!=3)
		BPS=1;
	    else
		switch(Get4()) {
		    case 1 :
			BPS=Get2(); break;
		    case 3 :
			Seek((long)Get4(),SEEK_SET);
			BPS=Get2(); if (BPS!=Get2() || BPS!=Get2()) BPS=0;

			break;
		    default :
			BPS=0; break;
		}
	}
	if (BPS==0) NoB_Error("bits per sample");

	if (PI==3) {
	    CMC=(1<<BPS)*3;
	    if (GetIFD(0x140)!=3 || Get4()!=CMC) NoB_Error("colormap");

	    CMB=Copy(Get4(),CMBC=CMC*2,"colormap"); ++tags;
	}
	if ((RPS=GetIFD4(0x116,IL))>IL)
	    RPS=IL;
	else
	    if (RPS==0) NoB_Error("rows per strip");

	ReadStrip(SPI=(IL+RPS-1)/RPS);

	if ((IC=GetIFD3(0x103,1))!=1 && IC!=5 && IC!=32773)
	    ERROR("unknown image compression method (warning).\n");

	if ((DP=GetIFD3(0x13d,1))!=1) {
	    if (DP!=2) ERROR("unknown differencing predictor (warning).\n");

	    ++tags;
	}
	if ((FO=GetIFD3(0x10a,1))!=1) {
	    if (FO!=2) ERROR("unknown fill order (warning).\n");

	    ++tags;
	}
	(void)fclose(file);

	if ((file=fopen(argv[argc-1],"wb"))==NULL) Error("file not open.");

	Nwrite((Byte *)MN,(size_t)4); Put4((Word4)8); Put2(tags);

	if (IW<65536) PutIFD3(0x100,(Word2)IW); else PutIFD4(0x100,1,IW);
	if (IL<65536) PutIFD3(0x101,(Word2)IL); else PutIFD4(0x101,1,IL);

	ofs=8+2+tags*12+4;
	if (PI==2) {
	    PutIFD(0x102,3,3); Put4(ofs); ofs+=6;
	}
	else {
	    PutIFD3(0x102,BPS);

	    if (PI==3) {
		CMO=ofs; ofs+=CMBC;
	    }
	}
	PutIFD3(0x103,IC);
	PutIFD3(0x106,PI);
	if (FO!=1) PutIFD3(0x10a,FO);

	if (len>1) {
	    PutIFD(0x10e,2,(Word4)len);
	    if (len<=4) {
		Nwrite((Byte *)ID,len); for (idx=len; idx<4; idx++) Put1(0);
	    }
	    else {
		Put4(ofs); ofs+=(Word4)len;
	    }
	}
	PutIFD4(0x111,SPI,ofs);
	PutIFD3(0x115,SPP);
	if (RPS<65536) PutIFD3(0x116,(Word2)RPS); else PutIFD4(0x116,1,RPS);

	PutIFD4(0x117,SPI,(SPI==1)?SBC(0):ofs+SPI*4);
        PutIFD3(0x11c,PC);
	if (DP!=1) PutIFD3(0x13d,DP);

	if (PI==3) {
	    PutIFD(0x140,3,CMC); Put4(CMO);
        }
	Put4((Word4)0);

	switch(PI) {
	    case 2 : Put2(BPS); Put2(BPS); Put2(BPS); break;
	    case 3 : Nwrite(CMB,(size_t)CMBC); break;
            default: break;
        }
	if (len>4) Nwrite((Byte *)ID,len);

	if (SPI>1) {
	    ofs+=(SPI*8);
	    for (idx=0; idx<SPI; idx++) {
		Put4(ofs); ofs+=SBC(idx);
	    }
	    for (idx=0; idx<SPI; idx++) Put4(SBC(idx));
	}
	for (idx=0; idx<SPI; idx++) Nwrite(SB(idx),(size_t)SBC(idx));

	(void)fclose(file); return 0;
}
