
#include <stdio.h>
#include <stdlib.h>
#include "rif_f.h"
#include "csi_f.h"
#include "sif_f.h"

extern void	Error(char *msg);

static int	Range(double x,double y,double z,double *min)
{
	double	max,xy=x+y,yz=y+z,zx=z+x,xyz=x+y+z;

	*min=max=0.0; if (x  <*min) *min=x;   else if (x  >max) max=x;
		      if (y  <*min) *min=y;   else if (y  >max) max=y;
		      if (z  <*min) *min=z;   else if (z  >max) max=z;
		      if (xy <*min) *min=xy;  else if (xy >max) max=xy;
		      if (yz <*min) *min=yz;  else if (yz >max) max=yz;
		      if (zx <*min) *min=zx;  else if (zx >max) max=zx;
		      if (xyz<*min) *min=xyz; else if (xyz>max) max=xyz;

	return (int)(max-(*min)+0.5);
}

#define LEN	2048

int	main(int argc,char **argv)
{
	char	**paths,form[LEN],path[LEN];
	int	Nx,Ny,Nz,Nh,Nv,Nd,k,j,i;
	double	hx,hy,hz,vx,vy,vz,dx,dy,dz,det,h0,v0,d0,pv,
		xh,xv,xd,yh,yv,yd,zh,zv,zd,x,y,z;
	FOM	***org,**new;

#ifdef	AFFINE
#define EXE	"affine_3d_float"
#else
#define EXE	"tli_float"
#endif
	if (argc!=12 && argc!=14 && argc!=17 && argc!=20) Error(
	    "usage :\n"
	    EXE " org/ NF hx hy hz vx vy vz dx dy dz\n"
	    EXE " org/ NF hx hy hz vx vy vz dx dy dz PV new/\n"
	    EXE " org/ NF hx hy hz vx vy vz dx dy dz h0 v0 d0 PV new/\n"
	    EXE " org/ NF hx hy hz vx vy vz dx dy dz h0 v0 d0 Nh Nv Nd PV new/"
	);
	paths=CheckSliceImages_Float(argv[1],argv[2],&Nx,&Ny,&Nz,NULL);

#define X	(double)Nx
#define Y	(double)Ny
#define Z	(double)Nz

	hx=atof(argv[3]); hy=atof(argv[ 4]); hz=atof(argv[ 5]);
	vx=atof(argv[6]); vy=atof(argv[ 7]); vz=atof(argv[ 8]);
	dx=atof(argv[9]); dy=atof(argv[10]); dz=atof(argv[11]);
	det=hx*vy*dz+vx*dy*hz+dx*hy*vz-dx*vy*hz-vx*hy*dz-hx*dy*vz;
	if (-0.5<X*Y*Z*det && X*Y*Z*det<0.5) Error("bad transfer matrix.");

	if (argc==12 || argc==14) {
	    Nh=Range(hx*X,hy*Y,hz*Z,&h0);
	    Nv=Range(vx*X,vy*Y,vz*Z,&v0);
	    Nd=Range(dx*X,dy*Y,dz*Z,&d0);

	    if (argc==12) {
		(void)printf("%lf\t%lf\t%lf\t%d\t%d\t%d\n",h0,v0,d0,Nh,Nv,Nd);
		goto Exit;
	    }
	}
	else {
	    h0=atof(argv[12]);
	    v0=atof(argv[13]);
	    d0=atof(argv[14]);

	    if (argc==17) {
		Nh=Nx;
		Nv=Ny;
		Nd=Nz;
	    }
	    else
		if ((Nh=atoi(argv[15]))<=0 ||
		    (Nv=atoi(argv[16]))<=0 ||
		    (Nd=atoi(argv[17]))<=0) Error("bad image size.");
	}
	if ((org=(FOM ***)malloc(sizeof(FOM **)*Nz))==NULL ||
	    (*org=(FOM **)malloc(sizeof(FOM *)*Nz*Ny))==NULL ||
	    (**org=(FOM *)malloc(sizeof(FOM)*Nz*Ny*Nx))==NULL ||
	    (new=(FOM **)malloc(sizeof(FOM *)*Nv))==NULL ||
	    (*new=(FOM *)malloc(sizeof(FOM)*Nv*Nh))==NULL)
	    Error("memory allocation error.");

	pv=atof(argv[argc-2]);

	for (k=0; k<Nz; k++)
{
	int	bps,nx,ny,n;
	FOM	**fom;
	char	*id,str[8][LEN],*u_c,
		*u="using integer PV as is floating-point PV",
		*c="converting PV by TIFF image description tag";
	double	min,max,b,s;

	if ((bps=ReadImageFile_Float(paths[k],&nx,&ny,&fom,&id))!=0) {
	    if (id!=NULL) {
		if ((n=sscanf(id,"%s %s %s %s %s %s %s %s",
				 str[0],str[1],str[2],str[3],
				 str[4],str[5],str[6],str[7]))==2 || n==6)
		    if (sscanf(str[n-2],"%lf",&min)!=1 ||
			sscanf(str[n-1],"%lf",&max)!=1 || min>=max)
			u_c=u;
		    else {
			u_c=c; b=min; s=(max-min)/(double)((1ULL<<bps)-1);
		    }
		else
		    u_c=((n!=3 && n!=4 && n!=8) ||
			 sscanf(str[n-2],"%lf",&b)!=1 ||
			 sscanf(str[n-1],"%lf",&s)!=1 || s<=0.0)?u:c;

		free(id);
	    }
	    else
		u_c=u;
#ifdef	U_C
	    (void)fprintf(stderr,"%s : %s (warning).\n",paths[k],u_c);
#endif
	}
	else {
	    u_c=NULL; if (id!=NULL) free(id);
	}
	if (u_c!=c) {
	    b=0.0; s=1.0;
	}
	if (k!=0) org[k]=org[k-1]+Ny;

	for (j=k==0; j<Ny; j++) org[k][j]=org[k][j-1]+Nx;

	for (j=0; j<ny; j++) {
	    for (i=0; i<nx; i++) org[k][j][i]=b+s*fom[j][i];

	    for (; i<Nx; i++) org[k][j][i]=pv;

	    free(fom[j]);
	}
	for (; j<Ny; j++)
	    for (i=0; i<Nx; i++) org[k][j][i]=pv;

	free(fom);
}
	for (j=1; j<Nv; j++) new[j]=new[j-1]+Nh;
{
	int	len=1;

	for (k=10; k<Nd; k*=10) ++len;
	(void)sprintf(form,"%s/%%0%dd.tif",argv[argc-1],len);
}
    h0+=0.5; xh=(vy*dz-vz*dy)/det; yh=(vz*dx-vx*dz)/det; zh=(vx*dy-vy*dx)/det;
    v0+=0.5; xv=(dy*hz-dz*hy)/det; yv=(dz*hx-dx*hz)/det; zv=(dx*hy-dy*hx)/det;
    d0+=0.5; xd=(hy*vz-hz*vy)/det; yd=(hz*vx-hx*vz)/det; zd=(hx*vy-hy*vx)/det;
    for (dx=xd*d0, dy=yd*d0, dz=zd*d0, k=0; k<Nd; k++, dx+=xd, dy+=yd, dz+=zd)
{
    for (vx=xv*v0, vy=yv*v0, vz=zv*v0, j=0; j<Nv; j++, vx+=xv, vy+=yv, vz+=zv)
    for (hx=xh*h0, hy=yh*h0, hz=zh*h0, i=0; i<Nh; i++, hx+=xh, hy+=yh, hz+=zh)
	if ((x=hx+vx+dx)<0.0 || x>=X ||
	    (y=hy+vy+dy)<0.0 || y>=Y ||
	    (z=hz+vz+dz)<0.0 || z>=Z)
	    new[j][i]=pv;
	else
#ifdef	AFFINE
	    new[j][i]=org[(int)z][(int)y][(int)x];
#else
{
	int	x1,x2,z1,y1,y2,z2;
	double	o,ox,oy,oz,xy,yz,zx;

	if ((x+=0.5)<1.0)
	    x1=x2=0;
	else if (x<X) {
	    x1=(x2=(int)x)-1; x-=(double)x2;
	}
	else {
	    x1=x2=Nx-1; x-=X;
	}
	if ((y+=0.5)<1.0)
	    y1=y2=0;
	else if (y<Y) {
	    y1=(y2=(int)y)-1; y-=(double)y2;
	}
	else {
	    y1=y2=Ny-1; y-=Y;
	}
	if ((z+=0.5)<1.0)
	    z1=z2=0;
	else if (z<Z) {
	    z1=(z2=(int)z)-1; z-=(double)z2;
	}
	else {
	    z1=z2=Nz-1; z-=Z;
	}
	o=org[z1][y1][x1]; ox=org[z1][y1][x2]-o;
			   oy=org[z1][y2][x1]-o; xy=org[z1][y2][x2]-ox-oy-o;
			   oz=org[z2][y1][x1]-o; yz=org[z2][y2][x1]-oy-oz-o;
						 zx=org[z2][y1][x2]-oz-ox-o;
	new[j][i]=(org[z2][y2][x2]-xy-yz-zx-ox-oy-oz-o)*x*y*z+
		  xy*x*y+yz*y*z+zx*z*x+
		  ox*x+oy*y+oz*z+
		  o;
}
#endif
	(void)sprintf(path,form,k);
	StoreImageFile_Float(path,Nh,Nv,new,SIF_F_desc);
	(void)printf("%d\t%s\n",k,SIF_F_desc);
}
	free(*new); free(new); free(**org); free(*org); free(org); 
Exit:;
	for (k=0; k<Nz; k++) free(paths[k]); free(paths);

	return 0;
}
